/******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2014 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*******************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without
any warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. FREESCALE neither guarantees nor will be
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED TO THE 
PROJECT BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES.
IN NO EVENT SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES 
ARISING OUT OF THIS AGREEMENT.
CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands
or actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result of the 
advise or assistance supplied CUSTOMER in connection with product, services
or goods supplied under this Agreement.
*******************************************************************************
* File: main.c
* Owner: b12257
* Version: 1.0
* Date: April-14-2014
* Classification General Business Information
* MMA8652FC example code with use of FRDM-KL25Z and FRDM-FXS-MULTI(B) boards
*******************************************************************************
* This code has been written for the Freescale FRDM-KL25Z + FRDM-FXS-MULTI-B 
* boards and demonstrates how to read the acceleration data from the MMA8652FC
* accelerometer by I2C using an interrupt technique.
* 
* The MMA8652FC is set to achieve the best noise performance: 
* ODR = 1.56Hz, High Resolution Mode
* 
* The calculated values can be also watched in the FreeMASTER application.
* 
* A detailed description of this example including hardware setup is available 
* on the sensors community web page at
* 
*******************************************************************************
Revision History:
Version		Date	 		Author		Description of Changes
1.0			April-14-2014	b12257		
******************************************************************************/

#include "derivative.h" 				// Include peripheral declarations 
#include "I2C.h"						// Include declarations of I2C communication routines 
#include "MMA8652FC.h"					// Include declarations of MMA8652FC registers 

/******************************************************************************
* Global variables and constants
******************************************************************************/

unsigned char AccelData[6];
short Xout_12_bit, Yout_12_bit, Zout_12_bit;
float Xout_g, Yout_g, Zout_g;

char DataReady;

/******************************************************************************
* Functions
******************************************************************************/

void MCU_Init(void);
void MMA8652FC_Init (void);
void MMA8652FC_Calibration (void);

/******************************************************************************
* Main
******************************************************************************/  

int main (void)
{
	DataReady = 0;
	MCU_Init();
  	MMA8652FC_Init();
  	MMA8652FC_Calibration();
  	   	  	
  	while(1)
    {
		if (DataReady)		// Is a new set of data ready? 
		{  		
			DataReady = 0;
																	
			I2C_ReadMultiRegisters(MMA8652FC_I2C_ADDRESS, OUT_X_MSB_REG, 6, AccelData);		// Read data output registers 0x01-0x06 
            
			// 12-bit accelerometer data
			Xout_12_bit = ((short) (AccelData[0]<<8 | AccelData[1])) >> 4;		// Compute 12-bit X-axis acceleration output value
			Yout_12_bit = ((short) (AccelData[2]<<8 | AccelData[3])) >> 4;		// Compute 12-bit Y-axis acceleration output value
			Zout_12_bit = ((short) (AccelData[4]<<8 | AccelData[5])) >> 4;		// Compute 12-bit Z-axis acceleration output value
			
			// Accelerometer data converted to g's 
			Xout_g = ((float) Xout_12_bit) / SENSITIVITY_2G;		// Compute X-axis output value in g's
			Yout_g = ((float) Yout_12_bit) / SENSITIVITY_2G;		// Compute Y-axis output value in g's
			Zout_g = ((float) Zout_12_bit) / SENSITIVITY_2G;		// Compute Z-axis output value in g's			
		}   		
	}
}

/******************************************************************************
* MCU initialization function
******************************************************************************/ 

void MCU_Init(void)
{
	//I2C1 module initialization
	SIM_SCGC4 |= SIM_SCGC4_I2C1_MASK;		// Turn on clock to I2C1 module 
	SIM_SCGC5 |= SIM_SCGC5_PORTC_MASK;		// Turn on clock to Port C module 
	PORTC_PCR1 |= PORT_PCR_MUX(0x2);		// PTC1 pin is I2C1 SCL line 
	PORTC_PCR2 |= PORT_PCR_MUX(0x2);		// PTC2 pin is I2C1 SDA line 
	I2C1_F  |= I2C_F_ICR(0x14); 			// SDA hold time = 2.125us, SCL start hold time = 4.25us, SCL stop hold time = 5.125us
	I2C1_C1 |= I2C_C1_IICEN_MASK;    		// Enable I2C1 module 
		
	//Configure the PTA5 pin (connected to the INT1 of the MMA8652FC) for falling edge interrupts
	SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK;		// Turn on clock to Port A module 
	PORTA_PCR5 |= (0|PORT_PCR_ISF_MASK|		// Clear the interrupt flag 
					PORT_PCR_MUX(0x1)|		// PTA5 is configured as GPIO 
					PORT_PCR_IRQC(0xA));	// PTA5 is configured for falling edge interrupts 
		
	//Enable PORTA interrupt on NVIC
	NVIC_ICPR |= 1 << ((INT_PORTA - 16)%32); 
	NVIC_ISER |= 1 << ((INT_PORTA - 16)%32);
}

/******************************************************************************
* MMA8652FC initialization function
******************************************************************************/ 

void MMA8652FC_Init (void)
{
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, CTRL_REG2, 0x40);		// Reset all registers to POR values
	
	Pause(0x631);		// ~1ms delay
	
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, XYZ_DATA_CFG_REG, 0x00);		// +/-2g range with ~0.977mg/LSB	
	
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, CTRL_REG2, 0x02);      // High Resolution mode
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, CTRL_REG3, 0x00);		// Push-pull, active low interrupt 
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, CTRL_REG4, 0x01);		// Enable DRDY interrupt 
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, CTRL_REG5, 0x01);		// DRDY interrupt routed to INT1 - PTA5 
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, CTRL_REG1, 0x39);		// ODR = 1.56Hz, Active mode	
}

/******************************************************************************
* Simple accelerometer offset calibration
******************************************************************************/ 

void MMA8652FC_Calibration (void)
{
	char X_offset, Y_offset, Z_offset;
	
	DataReady = 0;	
	while (!DataReady){}		// Is a first set of data ready? 
	DataReady = 0;
	
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, CTRL_REG1, 0x00);		// Standby mode	
	
	I2C_ReadMultiRegisters(MMA8652FC_I2C_ADDRESS, OUT_X_MSB_REG, 6, AccelData);		// Read data output registers 0x01-0x06  	
		
	Xout_12_bit = ((short) (AccelData[0]<<8 | AccelData[1])) >> 4;		// Compute 12-bit X-axis acceleration output value
	Yout_12_bit = ((short) (AccelData[2]<<8 | AccelData[3])) >> 4;		// Compute 12-bit Y-axis acceleration output value
	Zout_12_bit = ((short) (AccelData[4]<<8 | AccelData[5])) >> 4;		// Compute 12-bit Z-axis acceleration output value
		
	X_offset = Xout_12_bit / 2 * (-1);		// Compute X-axis offset correction value
	Y_offset = Yout_12_bit / 2 * (-1);		// Compute Y-axis offset correction value
	Z_offset = (Zout_12_bit - SENSITIVITY_2G) / 2 * (-1);		// Compute Z-axis offset correction value
		
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, OFF_X_REG, X_offset);		
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, OFF_Y_REG, Y_offset);	
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, OFF_Z_REG, Z_offset);	
		
	I2C_WriteRegister(MMA8652FC_I2C_ADDRESS, CTRL_REG1, 0x39);		// Active mode again	
}

/******************************************************************************
* PORT A Interrupt handler
******************************************************************************/ 

void PORTA_IRQHandler()
{
	PORTA_PCR5 |= PORT_PCR_ISF_MASK;			// Clear the interrupt flag 
	DataReady = 1;	
}
